if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "PRIEST" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.


C["Mind Flay"], _, C["Mind Flay Icon"] = GetSpellInfo(15407)
C["Shadow Word: Pain"], _, C["Shadow Word: Pain Icon"] = GetSpellInfo(589)
C["Vampiric Touch"], _, C["Vampiric Touch Icon"] = GetSpellInfo(34914)
C["Devouring Plague"], _, C["Devouring Plague Icon"] = GetSpellInfo(2944)
C["Mind Blast"], _, C["Mind Blast Icon"] = GetSpellInfo(8092)

-- Important Debuff and buff info
C["Shadow Weaving"]  = GetSpellInfo(15257)
C["Improved Scorch"] = GetSpellInfo(11095)
C["Improved Shadow Bolt"] = GetSpellInfo(18275)
C["Winter's Chill"]  = GetSpellInfo(11180)

-- Helper function to ensure SWP only cast when appropriate buffs and debuffs are up
function PriorityQueue:CheckSwpAuras()
   return select(3, self:GetBuffInfo(C["Shadow Weaving"])) >= 5
    and  (   select(3, self:GetDebuffInfo(C["Improved Scorch"]     )) >= 5
		  or select(3, self:GetDebuffInfo(C["Improved Shadow Bolt"])) >= 5
		  or select(3, self:GetDebuffInfo(C["Winter's Chill"]      )) >= 5)
end

---------------------------
-- Priest Priorities
---------------------------

function PriorityQueue:GetPriestPriorityDefaults()
	-- need a function to work out spec for now fixed values
	self.presets.priorities.name["Shadow"] =  L["Shadow"]
	self.presets.priorities.list["Shadow"] = { "b_swp", "vt", "mb", "dp", "mf", "none", "none", "none", "none", "none" }
	self.presets.default = "Shadow"
end

PriorityQueue.priorityTable.name["mf"] = C["Mind Flay"]
PriorityQueue.priorityTable.icon["mf"] = C["Mind Flay Icon"]
PriorityQueue.priorityTable.test["mf"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Mind Flay"])
	end 

-- basic check to see if appropriate debuffs are on target and buffs on caster
-- before casting SWP
PriorityQueue.priorityTable.name["b_swp"] = L["Buffed Shadow Word: Pain"]
PriorityQueue.priorityTable.icon["b_swp"] = C["Shadow Word: Pain Icon"]
PriorityQueue.priorityTable.test["b_swp"] =
	function () 
		return PriorityQueue:CheckSwpAuras() and not PriorityQueue:GetDebuffInfo(C["Shadow Word: Pain"])
		and PriorityQueue:SpellAvailable(C["Shadow Word: Pain"])
	end 

PriorityQueue.priorityTable.name["swp"] = C["Shadow Word: Pain"]
PriorityQueue.priorityTable.icon["swp"] = C["Shadow Word: Pain Icon"]
PriorityQueue.priorityTable.test["swp"] =
	function ()
		return not PriorityQueue:GetDebuffInfo(C["Shadow Word: Pain"])
		and PriorityQueue:SpellAvailable(C["Shadow Word: Pain"])
	end
	

PriorityQueue.priorityTable.name["vt"] = C["Vampiric Touch"]
PriorityQueue.priorityTable.icon["vt"] = C["Vampiric Touch Icon"]
PriorityQueue.priorityTable.test["vt"] = 
	function () 
		if not PriorityQueue:SpellAvailable(C["Vampiric Touch"]) then 
			return false
		end
		local vtdebuff, vtduration = PriorityQueue:GetDebuffInfo(C["Vampiric Touch"])
		return not vtdebuff or ( vtduration < self:GetExpectedCastTime(1.5) )
	end 

PriorityQueue.priorityTable.name["mb"] = C["Mind Blast"]
PriorityQueue.priorityTable.icon["mb"] = C["Mind Blast Icon"]
PriorityQueue.priorityTable.test["mb"] = 
	function () 
		return PriorityQueue:SpellAvailable(C["Mind Blast"])
	end 

PriorityQueue.priorityTable.name["dp"] = C["Devouring Plague"]
PriorityQueue.priorityTable.icon["dp"] = C["Devouring Plague Icon"]
PriorityQueue.priorityTable.test["dp"] = 
	function () 
		return not PriorityQueue:GetDebuffInfo(C["Devouring Plague"]) and PriorityQueue:SpellAvailable(C["Devouring Plague"])
	end 
	
