if not PriorityQueue then return end

if select(2, UnitClass('player')) ~= "WARLOCK" then
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PriorityQueue")
local C = PriorityQueue.constants -- Defined in PriorityQueue LUA no locale needed.

-- I hate typing a lot...

-- Helper functions

-- stores localized spell data in C table
-- name is spell name in english, id is spell id
local function spell(name, id)
	C[name], _, C[name .. " Icon"], C[name .. " Mana"], _, _, C[name .. " Cast Time"] = GetSpellInfo(id)
end

spell("Banish", 18647)
spell("Chaos Bolt", 59172)
spell("Conflagrate", 17962)
spell("Corruption", 47813)
spell("Curse of Agony", 47864)
spell("Curse of Doom", 47867)
spell("Curse of Elements", 47865)
spell("Curse of Tongues", 11719)
spell("Curse of Weakness", 50511)
spell("Drain Soul", 47855)
spell("Decimation", 63165)
spell("Fear", 6215)
-- spell("Siphon Life", 47862)
-- removed as a spell in 3.1
spell("Haunt", 59164)
spell("Immolate", 47811)
spell("Incinerate", 47838)
spell("Immolation Aura", 50589)
spell("Life Tap", 57946)
spell("Metamorphosis", 47241)
spell("Shadow Bolt", 47809)
spell("Soul Fire", 47825)
spell("Soulshatter", 29858)
spell("Unstable Affliction", 47843)

-- Important Buffs and Debuffs
spell("Shadow Embrace",32394)

C["Improved Scorch"] = GetSpellInfo(11095)
C["Improved Shadow Bolt"] = GetSpellInfo(18275)
C["Winter's Chill"]  = GetSpellInfo(11180)

-- Helper function for debuff related spells
-- x : Abbreviation for spell
-- y : Name of debuff
local function non_stacking_debuff(x, y)
	PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	local needed = C[y .. " Mana"]
	local casttime = C[y .. " Cast Time"] / 1000
	local endtime = 0
	-- each spell has a different flight speed, and the time depends on
	-- the distance to the target - we just assume half a second for
	-- all our dots
	local flighttime = 0.5
	PriorityQueue.priorityTable.test[x] = function () 
		local casting, _, _, _, _, t = UnitCastingInfo("player")
		-- It doesn't stack, so never cast if it's already casting.
		if casting == C[y] then
			endtime = t / 1000
			return false
		end
		if GetTime() - endtime < flighttime then
			return false
		end
		local mana = UnitPower("player")
		-- Instead of 1000, we should use the mana of the currently
		-- being cast spell...
		if mana < needed then return false end
		local dot, duration = PriorityQueue:GetDebuffInfo(C[y])
		if dot then
			if duration > PriorityQueue.db.char.priority.cooldown + casttime then
				return false
			end
		end
		return PriorityQueue:SpellAvailable(C[y])
	end 
end

local function curse_debuff(x, y)
    PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	local needed = C[y .. " Mana"]
	local casttime = C[y .. " Cast Time"] / 1000
	local endtime = 0
	local CurseDebuffList = { C["Curse of Agony"],    C["Curse of Doom"],
						C["Curse of Elements"], C["Curse of Tongues"],
						C["Curse of Weakness"]
					  }
					  				   
    PriorityQueue.priorityTable.test[x] = function ()
					  -- if any curse if on the target, return false
					  for _,v in ipairs(CurseDebuffList) do
                  		if PriorityQueue:GetDebuffInfo(v) 
						then return false end
                  end
                  
                  -- otherwise, check mana and if spell is available to be cast
                  local mana = UnitPower("player")
                  return mana >= needed and PriorityQueue:SpellAvailable(C[y])
    end
end

local function destruction(x, y)
	PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	local needed = C[y .. " Mana"]
	PriorityQueue.priorityTable.test[x] = function ()
		local mana = UnitPower("player")
		if mana < needed then return false end
		return PriorityQueue:SpellAvailable(C[y])
	end
end

local function conflag(x, y)
	PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	local needed = C[y .. " Mana"]
	PriorityQueue.priorityTable.test[x] = function ()
		local mana = UnitPower("player")
		if mana < needed then return false end
		return PriorityQueue:GetDebuffInfo(C["Immolate"])
			   and PriorityQueue:SpellAvailable(C[y])
	end
end

local function soulfire(x,y)
	local DECIMATION_CUTOFF = .35
	
	PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	local needed = C[y .. " Mana"]
	PriorityQueue.priorityTable.test[x] = function ()
		local mana = UnitPower("player")
		local  healthpercent = UnitHealth("target")/UnitHealthMax("target")
		
		if mana < needed or healthpercent > DECIMATION_CUTOFF 
			then return false 
		end
		return PriorityQueue:GetBuffInfo(C["Decimation"]) 
			   and PriorityQueue:SpellAvailable(C[y])
	end
end

local function drainsoul(x,y)
	local DRAINSOUL_CUTOFF = .25
	
    PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	PriorityQueue.priorityTable.test[x] = function ()
		local  healthpercent = UnitHealth("target")/UnitHealthMax("target")
		
		if healthpercent >  DRAINSOUL_CUTOFF then return false end
		
		return PriorityQueue:SpellAvailable(C[y])
	end
end

-- We only want to keep the life tap buff up when we have glyph and non ISL spec
-- TODO modify priority list appropriately	
local function lifetap(x, y)
	PriorityQueue.priorityTable.name[x] = C[y]
	PriorityQueue.priorityTable.icon[x] = C[y .. " Icon"]
	PriorityQueue.priorityTable.test[x] =
		 function ()                 -- return true if
	     	--local start, duration, enabled = GetSpellCooldown(C["Life Tap"]);
		 	return not PriorityQueue:GetBuffInfo(C[y])
		 end
end

-- Helper function to ensure Corruption only cast when appropriate buffs and debuffs are up
function PriorityQueue:CheckShadowAuras()
   return select(3, self:GetDebuffInfo(C["Shadow Embrace"])) == 2
    and  (   select(3, self:GetDebuffInfo(C["Improved Scorch"]     )) == 5
		  or select(3, self:GetDebuffInfo(C["Improved Shadow Bolt"])) == 5
		  or select(3, self:GetDebuffInfo(C["Winter's Chill"]      )) == 5)
end

-- Code for buffed Corruption
-- Because Corruption is refreshed with original crit and % dmg modifiers,
-- we only want to cast Corruption if shadow embrace and crit bonuses are up

PriorityQueue.priorityTable.name["b_c"] = L["Buffed Corruption"]
PriorityQueue.priorityTable.icon["b_c"] = C["Corruption"]
PriorityQueue.priorityTable.test["b_c"] =
	function ()
        
		return PriorityQueue:CheckShadowAuras()
			   and not PriorityQueue:GetBuffInfo(C["Corruption"])
		       and PriorityQueue:SpellAvailable(C["Corruption"])

	end


-- Improve to check duration remaining
PriorityQueue.priorityTable.name["banish"] = L["Banish Focus"]
PriorityQueue.priorityTable.icon["banish"] = C["Banish"]
PriorityQueue.priorityTable.test["banish"] =
	function ()     
		return not PriorityQueue:GetDebuffInfo(C["Banish"], true, "focus")
		       and PriorityQueue:SpellAvailable(C["Banish"])
	end	

-- Improve to check duration remaining
PriorityQueue.priorityTable.name["fear"] = L["Fear Focus"]
PriorityQueue.priorityTable.icon["fear"] = C["Fear"]
PriorityQueue.priorityTable.test["fear"] =
	function ()     
		return not PriorityQueue:GetDebuffInfo(C["Fear"], true, "focus")
		       and PriorityQueue:SpellAvailable(C["Fear"])
	end		
	
function PriorityQueue:GetWarlockPriorityDefaults()
	self.presets.priorities.name["Affliction Haunt/Ruin"] =  L["Affliction Haunt/Ruin"]
	self.presets.priorities.list["Affliction Haunt/Ruin"] = {"lt", "h", "ua", "coa", "b_c", "sb", "none", "none", "none", "none" }
	
	self.presets.priorities.name["Hybrid Imp/Conflag"] = L["Hybrid Imp/Conflag"]
 	self.presets.priorities.list["Hybrid Imp/Conflag"] = {"lt", "conf", "i", "sf", "coa", "c", "in", "none", "none" }

	
	self.presets.priorities.name["Hybrid Fel/Ember"] = L["Hybrid Fel/Ember"]
	self.presets.priorities.list["Hybrid Fel/Ember"] = {"lt", "i", "sf", "coa", "c", "in", "none", "none", "none", "none" }

	self.presets.priorities.name["Deep Destro"] = L["Deep Destro"]
	self.presets.priorities.list["Deep Destro"] = {"lt", "conf", "i", "sf", "cod", "cb", "in", "none", "none", "none" }
	
--	self.presets.priorities.name["Demon Meta/Ruin"] = L["Demon Meta/Ruin"]
--	self.presets.priorities.list["Demon Meta/Ruin"]  = {"lt", "meta", "ia", "i", "cod", "sb", "none", "none", "none", "none", "none" }
	
	self.presets.default = "Hybrid Imp/Conflag"

	self.db.char.priority.dropthreat.name = C["Soulshatter"]
	self.db.char.priority.dropthreat.icon = C["Soulshatter Icon"]

	non_stacking_debuff("h", "Haunt")
	non_stacking_debuff("ua", "Unstable Affliction")
	non_stacking_debuff("i", "Immolate")
	curse_debuff("coa", "Curse of Agony")
	curse_debuff("cod", "Curse of Doom")
	curse_debuff("coe", "Curse of Elements")
	curse_debuff("cow", "Curse of Weakness")
	curse_debuff("cot", "Curse of Tongues")
	non_stacking_debuff("c", "Corruption")
--	non_stacking_debuff("sl", "Siphon Life")
--  Removed for 3.1

	destruction("sb", "Shadow Bolt")
	destruction("in", "Incinerate")
	destruction("cb", "Chaos Bolt")
	soulfire("sf", "Soul Fire")
	conflag("conf", "Conflagrate")
	
	
--  Add in metamorphosis as meta
--  Add in Immolation Aura as ia

	lifetap("lt", "Life Tap")
end